/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "assert/assert.h"
#include "rpg/entity/entitydirection.h"
#include "symbol.h"
#include "game/time.h"

char_t symbolGetCharByEntity(const entity_t *ent) {
  assertNotNull(ent, "Entity cannot be NULL.");

  switch(ent->type) {
    case ENTITY_TYPE_SIGN:
      return '+';

    case ENTITY_TYPE_DOOR:
      return 'D';

    default:
      break;
  }

  switch(ent->direction) {
    case ENTITY_DIRECTION_EAST: return '>';
    case ENTITY_DIRECTION_WEST: return '<';
    case ENTITY_DIRECTION_NORTH: return '^';
    case ENTITY_DIRECTION_SOUTH: return 'v';
    default:
      assertUnreachable("Invalid entity direction.");
  }
}

char_t symbolGetCharByTile(const tile_t tile) {
  switch(tile) {
    case TILE_NULL:
      return ' ';

    case TILE_GRASS:
      return '#';

    case TILE_WATER:
      int32_t j = (int32_t)(TIME.time * 2) % 4;
      switch(j) {
        case 0: return '/';
        case 1: return '|';
        case 2: return '\\';
        case 3: return '|';
      }

    case TILE_BUILDING_WALL:
      return '-';

    case TILE_ROOF:
      return '/';

    default:
      assertUnreachable("Unknown char for tile.");
  }
}

uint8_t symbolGetColorByEntity(const entity_t *ent) {
  assertNotNull(ent, "Entity cannot be NULL.");
  switch(ent->type) {
    case ENTITY_TYPE_PLAYER:
      return COLOR_RED;

    case ENTITY_TYPE_NPC:
      return COLOR_MAGENTA;
    
    case ENTITY_TYPE_SIGN:
      return COLOR_YELLOW;

    case ENTITY_TYPE_DOOR:
      return COLOR_BROWN;

    default:
      assertUnreachable("Invalid entity type.");
  }
}

uint8_t symbolGetColorByTile(const tile_t tile) {
  switch(tile) {
    case TILE_NULL:
      return COLOR_BLACK;

    case TILE_GRASS:
      return COLOR_GREEN;

    case TILE_WATER:
      return COLOR_BLUE;

    case TILE_BUILDING_WALL:
      return COLOR_GRAY;

    case TILE_ROOF:
      return COLOR_BROWN;
    
    default:
      assertUnreachable("Unknown color for tile.");
  }
}