/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "rpg/world/map.h"

typedef enum {
  GAME_UPDATE_RESULT_CONTINUE = 0,
  GAME_UPDATE_RESULT_EXIT = 1
} gameupdateresult_t;

typedef enum {
  GAME_STATE_INITIAL = 0,
  GAME_STATE_OVERWORLD = 1,
  GAME_STATE_PAUSED = 2,
  GAME_STATE_MAP_CHANGE = 3
} gamestate_t;

typedef struct {
  map_t *currentMap;
  gamestate_t state;
  bool_t shouldExit;
  maplist_t mapNext;
} game_t;

extern game_t GAME;

/**
 * Initializes the game state.
 */
void gameInit();

/**
 * Updates the game state.
 * 
 * @param delta Time since last update.
 * @return Game update result, 0 for continue, 1 for exit, else for failure.
 */
gameupdateresult_t gameUpdate(const float_t delta);

/**
 * Cleans up the game state.
 */
void gameDispose();