/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "rpg/entity/entity.h"


typedef enum {
  TRIGGER_TYPE_NULL = 0
} triggertype_t;

typedef struct {
  uint16_t x, y;
  uint16_t width, height;
  uint8_t type;
} trigger_t;

/**
 * Initializes a trigger.
 * 
 * @param trigger Trigger to initialize.
 * @param type Type of trigger.
 * @param x X position of the trigger.
 * @param y Y position of the trigger.
 * @param width Width of the trigger.
 * @param height Height of the trigger.
 */
void triggerInit(
  trigger_t *trigger,
  const triggertype_t type,
  const uint16_t x,
  const uint16_t y,
  const uint16_t width,
  const uint16_t height
);

/**
 * Called by an entity that has stepped on the trigger.
 * 
 * @param trigger Trigger that was stepped on.
 * @param entity Entity that stepped on the trigger.
 */
void triggerOnStep(trigger_t *trigger, entity_t *entity);