/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "backbuffer.h"
#include "assert/assert.h"
#include "assert/assertgl.h"
#include "dawnopengl.h"

int32_t BACK_BUFFER_WIDTH;
int32_t BACK_BUFFER_HEIGHT;
float_t BACK_BUFFER_SCALE;

void backBufferSetSize(
  const int32_t width,
  const int32_t height,
  const float_t scale
) {
  assertTrue(width >= 0, "backBufferSetSize: Width must be >= 0");
  assertTrue(height >= 0, "backBufferSetSize: Height must be >= 0");
  assertTrue(scale > 0, "backBufferSetSize: Scale must be > 0");

  BACK_BUFFER_WIDTH = width;
  BACK_BUFFER_HEIGHT = height;
  BACK_BUFFER_SCALE = scale;
}

void backBufferBind() {
  glBindFramebuffer(GL_FRAMEBUFFER, 0);
  assertNoGLError();
  glViewport(0, 0, BACK_BUFFER_WIDTH, BACK_BUFFER_HEIGHT);
  assertNoGLError();
}

void backBufferClear() {
  glClearColor(0,0,0,1);
  assertNoGLError();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  assertNoGLError();
}