/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "display/color.h"

extern int32_t BACK_BUFFER_WIDTH;
extern int32_t BACK_BUFFER_HEIGHT;
extern float_t BACK_BUFFER_SCALE;

/**
 * Called by the host whenever the backbuffer size changes. Also includes a 
 * scale parameter to deal with highdpi screens.
 * 
 * @param width Width in pixels.
 * @param height Height in pixels.
 * @param scale Scale/Pixel density.
 */
void backBufferSetSize(
  const int32_t width,
  const int32_t height,
  const float_t scale
);

/**
 * Binds the backbuffer to the current framebuffer.
 */
void backBufferBind();

/**
 * Clears the backbuffer.
 * 
 * @param clearFlags Specifies rules on how to clear.
 * @param color Color to use when clearing (assuming you are clearing colors).
 */
void backBufferClear();