/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "font.h"

#ifndef FONT8X8_BASIC_H
  #include <font8x8_basic.h>
  #define FONT8X8_BASIC_H
#endif

font_t FONT;

void fontInit() {
  memset(&FONT, 0, sizeof(FONT));

  // font is uint8_t[8] * count of chars
  size_t charCount = sizeof(font8x8_basic) / 8;
  size_t totalPixels = charCount * 8 * 8;
  int32_t width = (int32_t)(totalPixels / 8);
  int32_t height = 8;

  textureInit(
    &FONT.texture,
    width,
    height,
    TEXTURE_FORMAT_R,
    TEXTURE_DATA_FORMAT_UNSIGNED_BYTE
  );
  FONT.texture.filterModeMag = TEXTURE_FILTER_MODE_NEAREST;
  FONT.texture.filterModeMin = TEXTURE_FILTER_MODE_NEAREST;

  uint8_t *pixels = (uint8_t*)malloc(totalPixels);

  for(size_t i = 0; i < charCount; i++) {
    for(size_t y = 0; y < 8; y++) {
      for(size_t x = 0; x < 8; x++) {
        uint8_t bit = (font8x8_basic[i][y] >> x) & 1;
        pixels[(i * 8) + (y * 8 * charCount) + x] = bit * 255;
      }
    }
  }

  textureBuffer(&FONT.texture, pixels);
  free(pixels);
}

void fontCharGetUV(const char_t c, vec4_t out) {
  size_t charCount = sizeof(font8x8_basic) / 8;
  int32_t index = (int32_t)c;
  if(index >= charCount) index = 0;

  int32_t x = index % charCount;
  int32_t y = 0;

  float_t width = (float_t)FONT.texture.width;
  float_t height = (float_t)FONT.texture.height;

  out[0] = (float_t)x * (8.0f / width);
  out[1] = (float_t)y * (8.0f / height);
  out[2] = out[0] + (8.0f / width);
  out[3] = out[1] + (8.0f / height);
}

void fontDispose() {
  textureDispose(&FONT.texture);
}