/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "quad.h"

void primitiveQuadBuffer(
  mesh_t *mesh,
  const vec2_t xy0,
  const vec2_t xy1,
  const vec2_t uv0,
  const vec2_t uv1,
  const color4f_t color,
  const size_t verticeIndex,
  const size_t indiceIndex
) {
  meshBufferPositions(
    mesh,
    (vec3_t[]){
      { xy0[0], xy0[1], 0.0f },
      { xy1[0], xy0[1], 0.0f },
      { xy1[0], xy1[1], 0.0f },
      { xy0[0], xy1[1], 0.0f }
    },
    verticeIndex,
    PRIMITIVE_QUAD_VERTICE_COUNT
  );

  primitiveQuadBufferCoordinates(mesh, uv0, uv1, verticeIndex);

  meshBufferIndices(
    mesh,
    (meshindice_t[]){
      verticeIndex + 0, verticeIndex + 1, verticeIndex + 2,
      verticeIndex + 2, verticeIndex + 3, verticeIndex + 0
    },
    indiceIndex,
    PRIMITIVE_QUAD_INDICE_COUNT
  );

  color4f_t colors[PRIMITIVE_QUAD_VERTICE_COUNT];
  color4fCopy(color, colors[0]);
  color4fCopy(color, colors[1]);
  color4fCopy(color, colors[2]);
  color4fCopy(color, colors[3]);
  
  
  meshBufferColors(
    mesh,
    colors,
    verticeIndex,
    PRIMITIVE_QUAD_VERTICE_COUNT
  );
}

void primitiveQuadBufferCoordinates(
  mesh_t *mesh,
  const vec2_t uv0,
  const vec2_t uv1,
  int32_t verticeIndex
) {
  meshBufferCoordinates(
    mesh,
    (vec2_t[]){
      { uv0[0], uv0[1] },
      { uv1[0], uv0[1] },
      { uv1[0], uv1[1] },
      { uv0[0], uv1[1] }
    },
    verticeIndex,
    PRIMITIVE_QUAD_VERTICE_COUNT
  );
}